package com.biganto.visual.roompark.presentation.screen.to_flat

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.FindFlatInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class FindFlatScreenPresenter @Inject constructor(
    private val interactor: FindFlatInteractor
)
    : BigantoBasePresenter<FindFlatScreen, FindFlatScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> FindFlatScreenViewState.SomeError(e)}


    override fun bindIntents() {

        val getFlatIntent = intent(FindFlatScreen::getFlat)
            .doOnNext{Timber.d(" flat is $it")}
            .flatMap { request ->
                interactor.getFlat(request.building,request.number)
                    .map<FindFlatScreenViewState> {FindFlatScreenViewState.FlatFounded(it)}
                    .onErrorReturn (::parseError)
            }
            .startWith(Observable.just(FindFlatScreenViewState.RequstFlatProgress()))


        val state = restoreStateObservable
            .mergeWith(getFlatIntent)
            .doOnError { Timber.e(it) }
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(FindFlatScreenViewState::class.java), FindFlatScreen::render)
    }
}